@@ -24,10 +24,10 @@ indii::Image::Image(const int width, const int height) :
 indii::Image::Image(const Image& img, const float scale) {
   const int width = std::floor(img.getWidth()/scale);
   const int height = std::floor(img.getHeight()/scale);
-     
-  r.resize(width, height, false);
-  g.resize(width, height, false);
-  b.resize(width, height, false);
+ 
+  r.resize(height, width, false);
+  g.resize(height, width, false);
+  b.resize(height, width, false);
  
   for (int x = 0; x < width; ++x) {
     for (int y = 0; y < height; ++y) {
