@@ -16,15 +16,17 @@
 #ifdef URBANLIGHTSCAPE
 #define PACKAGE_NAME "Urban Lightscape"
 #define PACKAGE_TARNAME "urbanlightscape"
-#define PACKAGE_VERSION "1.3.2"
-#define PACKAGE_DESC "A photo filter for exposure correction, localised brightness adjustments and artificial lighting."
+#define PACKAGE_VERSION "1.3.3"
+#define PACKAGE_DESC "A Photoshop plugin (and standalone application) for localised exposure correction and post-production lighting adjustments."
+#define PACKAGE_ADOBE_ID "425"
 #endif
 
 #ifdef TINTII
 #define PACKAGE_NAME "Tintii"
 #define PACKAGE_TARNAME "tintii"
-#define PACKAGE_VERSION "2.8.2"
-#define PACKAGE_DESC "A photo filter for selective colour effects, and adjustments of hue, saturation and lightness."
+#define PACKAGE_VERSION "2.9.0"
+#define PACKAGE_DESC "A Photoshop plugin (and standalone application) for saturation control, especially selective colour effects."
+#define PACKAGE_ADOBE_ID "421"
 #endif
 
 #endif
