@@ -1,5 +1,5 @@
 /**
- * Copyright (C) 2007-2013 Lawrence Murray
+ * Copyright (C) 2007-2014 Lawrence Murray
  *
  * This program is free software; you can redistribute it and/or modify it
  * under the terms of the GNU General Public License as published by the Free
@@ -14,9 +14,6 @@
 
 #include "ClusterModelObserver.hpp"
 
-//#include "wx/app.h"
-//#include "wx/busyinfo.h"
-
 #include <cassert>
 #include <ctime>
 #include <limits>
@@ -27,7 +24,7 @@ using namespace indii;
 using namespace std;
 
 ClusterModel::ClusterModel(ImageResource* res) :
-    Model(res), minClusterer(NULL), locked(false) {
+    Model(res), clusterer(time(NULL)), locked(false) {
   this->saturationThreshold = 0.0625f;
   this->maxPixels = 50000;
   this->reps = 12;
@@ -37,16 +34,16 @@ ClusterModel::ClusterModel(ImageResource* res) :
   this->centroidDecay = 0.0f;
   this->saturationSoftness = 0.125f;
   this->centroidSoftness = 0.0f;
-  this->colours.resize(k, *wxWHITE);
+  this->colours.resize(k);
+  this->mus.resize(k);
   this->hues.resize(k, 0.0f);
   this->sats.resize(k, -1.0f);
   this->lights.resize(k, 0.0f);
   this->alphas.resize(k, 1.0f);
-  this->minClusterer = new KMeansClusterer<>(k, time(NULL));
 }
 
 ClusterModel::~ClusterModel() {
-  delete minClusterer;
+  //
 }
 
 void ClusterModel::setDefaults() {
@@ -81,6 +78,7 @@ void ClusterModel::setNumClusters(const unsigned k) {
   if (this->k != k) {
     this->k = k;
     colours.resize(k, *wxWHITE);
+    mus.resize(k);
     hues.resize(k);
     sats.resize(k);
     lights.resize(k);
@@ -97,27 +95,27 @@ void ClusterModel::setNumClusters(const unsigned k) {
 }
 
 void ClusterModel::setHard(const float hard) {
-  /* pre-condition */
-  assert(hard >= 0.0f && hard <= 1.0f);
-
-  if (this->hard != hard) {
-    this->hard = hard;
+  float hard1 = bound(0.0f, hard, 1.0f);
+  if (this->hard != hard1) {
+    this->hard = hard1;
     notifyAll();
     notifyIsHardChange();
   }
 }
 
 void ClusterModel::setNumRepetitions(const unsigned reps) {
-  if (this->reps != reps) {
-    this->reps = reps;
+  unsigned reps1 = bound(1u, reps, 20u);
+  if (this->reps != reps1) {
+    this->reps = reps1;
     notifyAll();
     notifyNumRepetitionsChange();
   }
 }
 
 void ClusterModel::setSaturationThreshold(const float x) {
-  if (this->saturationThreshold != x) {
-    this->saturationThreshold = x;
+  float x1 = bound(0.0f, x, 1.0f);
+  if (this->saturationThreshold != x1) {
+    this->saturationThreshold = x1;
     prepare();
     notifyAll();
     notifySaturationThresholdChange();
@@ -125,8 +123,9 @@ void ClusterModel::setSaturationThreshold(const float x) {
 }
 
 void ClusterModel::setMaxPixels(const unsigned n) {
-  if (this->maxPixels != n) {
-    this->maxPixels = n;
+  unsigned n1 = bound(1000, n, 1e6);
+  if (this->maxPixels != n1) {
+    this->maxPixels = n1;
     prepare();
     notifyAll();
     notifyMaxPixelsChange();
@@ -134,126 +133,124 @@ void ClusterModel::setMaxPixels(const unsigned n) {
 }
 
 void ClusterModel::setSaturationDecay(const float x) {
-  /* pre-condition */
-  assert(x >= 0.0f && x <= 1.0f);
-
-  if (this->saturationDecay != x) {
-    this->saturationDecay = x;
+  float x1 = bound(0.0f, x, 1.0f);
+  if (this->saturationDecay != x1) {
+    this->saturationDecay = x1;
     notifyAll();
     notifySaturationDecayChange();
   }
 }
 
 void ClusterModel::setCentroidDecay(const float x) {
-  /* pre-condition */
-  assert(x >= 0.0f && x <= 1.0f);
-
-  if (this->centroidDecay != x) {
-    this->centroidDecay = x;
+  float x1 = bound(0.0f, x, 1.0f);
+  if (this->centroidDecay != x1) {
+    this->centroidDecay = x1;
     notifyAll();
     notifyCentroidDecayChange();
   }
 }
 
 void ClusterModel::setSaturationSoftness(const float x) {
-  /* pre-condition */
-  assert(x >= 0.0 && x <= 1.0f);
-
-  if (this->saturationSoftness != x) {
-    this->saturationSoftness = x;
+  float x1 = bound(0.0f, x, 1.0f);
+  if (this->saturationSoftness != x1) {
+    this->saturationSoftness = x1;
     notifyAll();
     notifySaturationSoftnessChange();
   }
 }
 
 void ClusterModel::setCentroidSoftness(const float x) {
-  /* pre-condition */
-  assert(x >= 0.0 && x <= 1.0f);
-
-  if (this->centroidSoftness != x) {
-    this->centroidSoftness = x;
+  float x1 = bound(0.0f, x, 1.0f);
+  if (this->centroidSoftness != x1) {
+    this->centroidSoftness = x1;
     notifyAll();
     notifyCentroidSoftnessChange();
   }
 }
 
 void ClusterModel::setColour(const unsigned i, const wxColour& col) {
-  /* pre-condition */
-  assert(i < k);
+  if (i < k) {
+    value_type x(3);
+    x(0) = col.Red() / 255.0f;
+    x(1) = col.Green() / 255.0f;
+    x(2) = col.Blue() / 255.0f;
+    distance_type::prepare(x);
+    colours[i] = col;
+    mus[i] = x;
+  }
+}
 
-  ClusterVector<>::type x(ClusterVector<>::N);
-  x(0) = col.Red()/255.0f;
-  x(1) = col.Green()/255.0f;
-  x(2) = col.Blue()/255.0f;
-  indii::KMeansClusterer<>::distance_type::prepare(x);
+void ClusterModel::setColour(const unsigned i, const float r, const float g, const float b) {
+  if (i < k) {
+    //float r1 = bound(0.0f, r, 1.0f);
+    //float g1 = bound(0.0f, g, 1.0f);
+    //float b1 = bound(0.0f, b, 1.0f);
 
-  colours[i] = col;
-  minClusterer->setCentroid(i, x);
+    value_type x(3);
+    x(0) = r;
+    x(1) = g;
+    x(2) = b;
+    //distance_type::prepare(x);
+
+    wxColour col;
+    col.Set(255.0f*r, 255.0f*g, 255.0f*b);
+  
+    colours[i] = col;
+    mus[i] = x;
+  }
 }
 
 void ClusterModel::setHue(const unsigned i, const float x) {
-  /* pre-condition */
-  assert(i < k);
-  assert(x >= -0.5f && x <= 0.5f);
-
-  if (hues[i] != x) {
-    hues[i] = x;
-    notifyAll();
-    notifyHueChange(i);
+  if (i < k) {
+    float x1 = bound(-0.5f, x, 0.5f);
+    if (hues[i] != x1) {
+      hues[i] = x1;
+      notifyAll();
+      notifyHueChange(i);
+    }
   }
 }
 
 void ClusterModel::setSat(const unsigned i, const float x) {
-  /* pre-condition */
-  assert(i < k);
-  assert(x >= -1.0f && x <= 1.0f);
-
-  if (sats[i] != x) {
-    sats[i] = x;
-    notifyAll();
-    notifySatChange(i);
+  if (i < k) {
+    float x1 = bound(-1.0f, x, 1.0f);
+    if (sats[i] != x1) {
+      sats[i] = x1;
+      notifyAll();
+      notifySatChange(i);
+    }
   }
 }
 
 void ClusterModel::setLight(const unsigned i, const float x) {
-  /* pre-condition */
-  assert(i < k);
-  assert(x >= -1.0f && x <= 1.0f);
-
-  if (lights[i] != x) {
-    lights[i] = x;
-
-    /* notify observers */
-    notifyAll();
-    notifyLightChange(i);
+  if (i < k) {
+    float x1 = bound(-1.0f, x, 1.0f);
+    if (lights[i] != x1) {
+      lights[i] = x1;
+      notifyAll();
+      notifyLightChange(i);
+    }
   }
 }
 
 void ClusterModel::setAlpha(const unsigned i, const float x) {
-  /* pre-condition */
-  assert(i < k);
-  assert(x >= 0.0f && x <= 1.0f);
-
-  if (alphas[i] != x) {
-    alphas[i] = x;
-
-    /* notify observers */
-    notifyAll();
-    notifyAlphaChange(i);
+  if (i < k) {
+    float x1 = bound(0.0f, x, 1.0f);
+    if (alphas[i] != x1) {
+      alphas[i] = x1;
+      notifyAll();
+      notifyAlphaChange(i);
+    }
   }
 }
 
 void ClusterModel::show(const unsigned i, const bool on) {
-  /* pre-condition */
-  assert(i < k);
-
-  if (isShown(i) != on) {
+  if (i < k && isShown(i) != on) {
     if (on) {
       sats[i] = 0.0f;
     } else {
       sats[i] = -1.0f;
     }
-
     notifyAll();
     notifySatChange(i);
   }
@@ -273,7 +270,7 @@ void ClusterModel::calcFg(const wxRect& rect, const float scale, Image& o) {
 
   Image& img = *res->get(scale);
 
-  #pragma omp parallel
+#pragma omp parallel
   {
     ColourSpace::rgb_t rgb(3);
     ColourSpace::hsl_t hsl(3);
@@ -282,7 +279,7 @@ void ClusterModel::calcFg(const wxRect& rect, const float scale, Image& o) {
     int x, y, cluster;
     float decay, light, sat;
 
-    #pragma omp for schedule(dynamic)
+#pragma omp for
     for (y = 0; y < rect.height; ++y) {
       for (x = 0; x < rect.width; ++x) {
         rgb(0) = img.r(rect.y + y, rect.x + x);
@@ -290,27 +287,25 @@ void ClusterModel::calcFg(const wxRect& rect, const float scale, Image& o) {
         rgb(2) = img.b(rect.y + y, rect.x + x);
         cs.rgb2hsl(rgb, hsl);
         decay = threshold(hsl(1), saturationDecay, saturationSoftness);
-
+        
         /* cluster adjustments */
-        indii::KMeansClusterer<>::distance_type::prepare(rgb);
+        distance_type::prepare(rgb);
+        cluster = clusterer.assign(rgb, ds, mus);
         if (isHard()) {
-          cluster = minClusterer->assign(rgb);
-
           /* hue */
           hsl(0) = fmod(1.0f + hsl(0) + hues[cluster], 1.0f);
 
           /* saturation */
-          sat = decay * sats[cluster];
+          sat = sats[cluster];
           if (sat >= 0.0f) {
             hsl(1) /= 1.0f - sat;
           } else {
             hsl(1) *= 1.0f + sat;
           }
-          hsl(1) = bound(0.0f, decay * hsl(1), 1.0f);
-          //hsl(1) = bound(0.0f, decay*(sats[cluster] + 1.0f)*hsl(1), 1.0f);
+          hsl(1) = bound(0.0f, decay*hsl(1), 1.0f);
 
           /* lightness */
-          light = lights[cluster];
+          light = decay*lights[cluster];
           if (light >= 0.0f) {
             hsl(2) /= 1.0f - light;
           } else {
@@ -321,8 +316,6 @@ void ClusterModel::calcFg(const wxRect& rect, const float scale, Image& o) {
           /* alpha */
           //a = bound(0.0f, decay*alphas[cluster], 255.0f);
         } else {
-          minClusterer->distances(rgb, ds);
-
           unsigned i;
           float h = 0.0f, s = 0.0f, l = 0.0f/*, a = 0.0f*/, D = 0.0f;
           for (i = 0; i < k; ++i) {
@@ -345,17 +338,16 @@ void ClusterModel::calcFg(const wxRect& rect, const float scale, Image& o) {
           hsl(0) = fmod(1.0f + hsl(0) + h, 1.0f);
 
           /* saturation */
-          sat = decay * s;
+          sat = s;
           if (sat >= 0.0f) {
             hsl(1) /= 1.0f - sat;
           } else {
             hsl(1) *= 1.0f + sat;
           }
-          hsl(1) = bound(0.0f, decay * hsl(1), 1.0f);
-          //hsl(1) = bound(0.0f, decay*(s + 1.0f)*hsl(1), 1.0f);
+          hsl(1) = bound(0.0f, decay*hsl(1), 1.0f);
 
           /* lightness */
-          light = l;
+          light = decay*l;
           if (light >= 0.0f) {
             hsl(2) /= 1.0f - light;
           } else {
@@ -390,22 +382,24 @@ void ClusterModel::calcAlpha(const unsigned i, const wxRect& rect,
   {
     ColourSpace::rgb_t rgb(3);
     ColourSpace::hsl_t hsl(3);
-    ClusterVector<>::type pixel(ClusterVector<>::N);
+    value_type pixel(3);
+    std::vector<float> ds(k);
+    int x, y, cluster;
 
-    int x, y;
-
-#pragma omp for schedule(dynamic)
+#pragma omp for
     for (y = 0; y < rect.height; ++y) {
       for (x = 0; x < rect.width; ++x) {
         rgb(0) = img->r(rect.y + y, rect.x + x);
         rgb(1) = img->g(rect.y + y, rect.x + x);
         rgb(2) = img->b(rect.y + y, rect.x + x);
         cs.rgb2hsl(rgb, hsl);
-        pixel = rgb;
+        pixel(0) = rgb(0);
+        pixel(1) = rgb(1);
+        pixel(2) = rgb(2);
 
-        /* cluster adjustments */
-        indii::KMeansClusterer<>::distance_type::prepare(pixel);
-        if (minClusterer->assign(pixel) == i) {
+        distance_type::prepare(pixel);
+        cluster = clusterer.assign(pixel, ds, mus);
+        if (cluster == i) {
           c(y, x) = threshold(hsl(1), saturationDecay, saturationSoftness);
         } else {
           c(y, x) = 0;
@@ -423,14 +417,15 @@ sparse_mask ClusterModel::calcMask(const unsigned i, const wxRect& rect,
   sparse_mask m(rect.height, rect.width);
   Image* img = res->get(scale);
 
-#pragma omp parallel default(shared)
+#pragma omp parallel
   {
     ColourSpace::rgb_t rgb(3);
     ColourSpace::hsl_t hsl(3);
-    ClusterVector<>::type pixel(ClusterVector<>::N);
-    int x, y;
+    value_type pixel(3);
+    std::vector<float> ds(k);
+    int x, y, cluster;
 
-#pragma omp for schedule(dynamic)
+#pragma omp for
     for (y = 0; y < rect.height; ++y) {
       for (x = 0; x < rect.width; ++x) {
         rgb(0) = img->r(rect.y + y, rect.x + x);
@@ -439,8 +434,9 @@ sparse_mask ClusterModel::calcMask(const unsigned i, const wxRect& rect,
         cs.rgb2hsl(rgb, hsl);
         pixel = rgb;
 
-        indii::KMeansClusterer<>::distance_type::prepare(pixel);
-        if (minClusterer->assign(pixel) == i) {
+        distance_type::prepare(pixel);
+        cluster = clusterer.assign(pixel, ds, mus);
+        if (cluster == i) {
 #pragma omp critical
           {
             /* sparse matrix, so need to control access, elements may be
@@ -463,22 +459,25 @@ void ClusterModel::prepare() {
   int originalWidth = res->getWidth();
   int originalHeight = res->getHeight();
   int pixels = originalWidth * originalHeight;
-  float scale = static_cast<float>(pixels) / maxPixels;
+  float scale = static_cast<float>(pixels)/maxPixels;
   Image* working = res->getLow(scale);
   int workingWidth = working->getWidth();
   int workingHeight = working->getHeight();
+  data.clear();
 
   /* prepare data set for clustering */
-  data.clear();
 #pragma omp parallel
   {
-    ClusterVector<>::type pixel(ClusterVector<>::N);
+    value_type pixel(3);
+    std::vector<float> ds(k);
+    std::vector<value_type> data1;
     int x, y;
     float s;
+    data1.reserve(workingWidth * workingHeight);
 
 #pragma omp for
-    for (x = 0; x < workingWidth; ++x) {
-      for (y = 0; y < workingHeight; ++y) {
+    for (y = 0; y < workingHeight; ++y) {
+      for (x = 0; x < workingWidth; ++x) {
         pixel(0) = working->r(y, x);
         pixel(1) = working->g(y, x);
         pixel(2) = working->b(y, x);
@@ -486,66 +485,57 @@ void ClusterModel::prepare() {
         /* saturation threshold */
         s = saturation(pixel(0), pixel(1), pixel(2));
         if (s >= saturationThreshold) {
-          indii::KMeansClusterer<>::distance_type::prepare(pixel);
-#pragma omp critical
-          {
-            data.add(pixel);
-          }
+          distance_type::prepare(pixel);
+          data1.push_back(pixel);
         }
       }
     }
+
+#pragma omp critical
+    {
+      data.insert(data.end(), data1.begin(), data1.end());
+    }
   }
 }
 
 void ClusterModel::cluster() {
+  const int maxIters = 100;
   if (locked) {
     return;
   }
-  const unsigned MAX_ITERS = 100;
-  const unsigned seed = time(NULL);
-  float minError = std::numeric_limits<float>::max();
 
-#pragma omp parallel shared(minError)
+//#pragma omp parallel
   {
-    /* cluster */
-    KMeansClusterer<>* clusterer;
-    float error;
+    value_type pixel(3);
+    std::vector<value_type> mus(k);
+    vector_type ds(data.size());
+    int_vector_type ks(data.size()), cs(k);
+    float err, minErr;
     int i;
-    ClusterVector<>::type pixel(ClusterVector<>::N);
 
-#pragma omp for schedule(dynamic)
-    for (i = 0; i < (int) reps; i++) {
-      clusterer = new KMeansClusterer<>(k, seed + i);
-      clusterer->cluster(data, MAX_ITERS);
-      error = clusterer->getError();
-#pragma omp critical
-      {
-        if (error < minError) {
-          minError = error;
-          delete minClusterer;
-          minClusterer = clusterer;
-        } else {
-          delete clusterer;
-        }
-        //wxYield();
+//#pragma omp for
+    for (i = 0; i < (int) reps; ++i) {
+      clusterer.cluster(data, ks, ds, mus, cs, maxIters);
+      err = ublas::norm_2(ds);
+      if (i == 0 || err < minErr) {
+        minErr = err;
+        this->mus = mus;
       }
     }
-    assert(minClusterer != NULL);
+  //}
+  }
 
-    /* cluster colours */
-    ColourSpace cs;
-    ColourSpace::rgb_t rgb(3);
-    ColourSpace::hsl_t hsl(3);
-#pragma omp for
-    for (i = 0; i < (int) k; i++) {
-      pixel = 0.5f * minClusterer->getCentroid(i)
-          + ublas::scalar_vector<float>(3, 0.5f);
-      rgb = pixel;
-      cs.rgb2hsl(rgb, hsl);
-      hsl(1) = 0.8f;
-      hsl(2) = 0.8f;
-      cs.hsl2rgb(hsl, rgb);
-      colours[i].Set(uround(rgb(0)), uround(rgb(1)), uround(rgb(2)));
-    }
+  ColourSpace cs;
+  ColourSpace::rgb_t rgb(3);
+  ColourSpace::hsl_t hsl(3);
+  value_type pixel(3);
+  for (int i = 0; i < (int) k; ++i) {
+    pixel = 0.5f * mus[i] + ublas::scalar_vector<float>(3, 0.5f);
+    rgb = pixel;
+    cs.rgb2hsl(rgb, hsl);
+    hsl(1) = 0.8f;
+    hsl(2) = 0.8f;
+    cs.hsl2rgb(hsl, rgb);
+    colours[i].Set(uround(rgb(0)), uround(rgb(1)), uround(rgb(2)));
   }
 }
