@@ -16,11 +16,13 @@
 #include "ClusterModelObservable.hpp"
 #include "../../model/Model.hpp"
 #include "../../image/ImageManipulation.hpp"
-#include "../../cluster/DataSet.hpp"
 #include "../../cluster/KMeansClusterer.hpp"
+#include "../../cluster/PearsonDistance.hpp"
 
 #include "wx/colour.h"
 
+#include "boost/numeric/ublas/vector.hpp"
+
 #include <vector>
 
 namespace indii {
@@ -34,6 +36,32 @@ class ClusterModel: public Model, public ClusterModelObservable {
 public:
   using Model::calcFg;
 
+  /**
+   * Value type for clusterer.
+   */
+  typedef boost::numeric::ublas::vector<float,
+      boost::numeric::ublas::bounded_array<float, 3> > value_type;
+
+  /**
+   * Distance type for clusterer.
+   */
+  typedef PearsonDistance<value_type, float> distance_type;
+
+  /**
+   * Clusterer type.
+   */
+  typedef KMeansClusterer<distance_type> clusterer_type;
+
+  /**
+   * Vector type.
+   */
+  typedef boost::numeric::ublas::vector<float> vector_type;
+
+  /**
+   * Integer vector type.
+   */
+  typedef boost::numeric::ublas::vector<int> int_vector_type;
+
   /**
    * Constructor.
    *
@@ -210,6 +238,16 @@ public:
    */
   const wxColour& getColour(const unsigned i);
 
+  /**
+   * Get cluster colour.
+   *
+   * @param i Cluster number.
+   * @param r Red value.
+   * @param g Green value.
+   * @param b Blue value.
+   */
+  void getColour(const unsigned i, float* r, float* g, float* b);
+  
   /**
    * Set cluster colour.
    *
@@ -218,6 +256,16 @@ public:
    */
   void setColour(const unsigned i, const wxColour& col);
 
+  /**
+   * Set cluster colour.
+   *
+   * @param i Cluster number.
+   * @param r Red value.
+   * @param g Green value.
+   * @param b Blue value.
+   */
+  void setColour(const unsigned i, const float r, const float g, const float b);
+
   /**
    * Get cluster hue rotation.
    *
@@ -428,14 +476,19 @@ private:
   std::vector<wxColour> colours;
 
   /**
-   * Data set for clustering.
+   * Cluster means.
    */
-  indii::DataSet<float, unsigned> data;
+  std::vector<value_type> mus;
 
   /**
-   * Clusterer used for last clustering.
+   * Clustering data set.
    */
-  indii::KMeansClusterer<>* minClusterer;
+  std::vector<value_type> data;
+
+  /**
+   * Clusterer.
+   */
+  clusterer_type clusterer;
 
   /**
    * Is model locked?
@@ -495,6 +548,15 @@ inline const wxColour& indii::ClusterModel::getColour(const unsigned i) {
   return colours[i];
 }
 
+inline void indii::ClusterModel::getColour(const unsigned i, float* r, float* g, float* b) {
+  /* pre-condition */
+  assert(i < k);
+  
+  *r = mus[i][0];
+  *g = mus[i][1];
+  *b = mus[i][2];
+}
+
 inline float indii::ClusterModel::getHue(const unsigned i) {
   /* pre-condition */
   assert(i < k);
